import merge from 'lodash.merge';

var vbButtonClose = {
  name: 'vbButtonClose',
  functional: true,
  render: function render(h, _ref) {
    var listeners = _ref.listeners;

    var content = h('span', {
      attrs: {
        'aria-hidden': true
      },
      domProps: {
        innerHTML: '&times;'
      }
    });

    return h('button', {
      attrs: {
        type: 'button'
      },
      'class': ['close-button'],
      on: {
        click: typeof listeners['click'] === 'function' ? listeners.click : function () {}
      }
    }, [content]);
  }
};

/**
 * Register a component plugin as being loaded.
 *
 * @param {object} Vue
 * @param {string} name - Component name
 * @param {object} def - Component definition
 */
function registerComponent(Vue, name, def) {
  Vue._vue_bedrock_components_ = Vue._vue_bedrock_components_ || {};

  if (!Vue._vue_bedrock_components_[name] && def && name) {
    Vue._vue_bedrock_components_[name] = true;
    Vue.component(name, def);
  }
}

/**
 * Install plugin if window.Vue available
 * @param {object} VuePlugin - Plugin definition
 */
function vueUse(VuePlugin) {
  if (typeof window !== 'undefined' && window.Vue) {
    window.Vue.use(VuePlugin);
  }
}

var VuePlugin = {
  install: function install(Vue) {
    registerComponent(Vue, 'vbButtonClose', vbButtonClose);
  }
};

vueUse(VuePlugin);

/**
 * Prefix used when generating some internal properties.
 * @const {string}
 */

/**
 * Array of common component colouring variants.
 * @type {string[]}
 */
var variants = ['primary', 'secondary', 'success', 'warning', 'alert'];

var sizes = ['small', 'large'];

var vbCallout = {
  name: 'vbCallout',
  render: function render(h) {
    if (!this.localVisible) return h(false);

    var closeBtn = h('vbButtonClose', {
      on: {
        click: this.close
      }
    });

    return h('div', {
      'class': ['callout', this.computedColoringClass, this.computedSizeClass]
    }, [this.closable ? closeBtn : null, this.$slots.default]);
  },

  components: {
    vbButtonClose: vbButtonClose
  },
  props: {
    closable: Boolean,
    coloring: {
      type: String,
      validator: function validator(variant) {
        return variants.includes(variant);
      }
    },
    size: {
      type: String,
      validator: function validator(size) {
        return sizes.includes(size);
      }
    }
  },
  data: function data() {
    return {
      localVisible: true
    };
  },

  computed: {
    computedColoringClass: function computedColoringClass() {
      return typeof this.coloring === 'string' ? this.coloring : null;
    },
    computedSizeClass: function computedSizeClass() {
      return typeof this.size === 'string' ? this.size : null;
    }
  },
  methods: {
    close: function close(e) {
      e.preventDefault();
      e.stopPropagation();

      this.localVisible = false;
    }
  }
};

var VuePlugin$1 = {
  install: function install(Vue) {
    registerComponent(Vue, 'vbCallout', vbCallout);
  }
};

vueUse(VuePlugin$1);

/**
 * SSR Safe Client Side ID attribute generation
 *
 * Based on BootstrapVue's ID mixin:
 * @see https://raw.githubusercontent.com/bootstrap-vue/bootstrap-vue/v2.0.0-rc.6/src/mixins/id.js
 */

var IdMixin = {
  props: {
    id: {
      type: String,
      default: null
    }
  },
  methods: {
    safeId: function safeId() {
      var suffix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

      var id = this.id || this.localId_ || null;

      if (!id) {
        return null;
      }

      suffix = String(suffix).replace(/\s+/g, '_');

      return suffix ? id + '_' + suffix : id;
    }
  },
  computed: {
    localId_: function localId_() {
      if (!this.$isServer && !this.id && typeof this._uid !== 'undefined') {
        return '__VBID__' + this._uid;
      }
    }
  }
};

// Production steps of ECMA-262, Edition 6, 22.1.2.1
// es6-ified by @alexsasharegan
if (!Array.from) {
  Array.from = function () {
    var toStr = Object.prototype.toString;
    var isCallable = function isCallable(fn) {
      return typeof fn === 'function' || toStr.call(fn) === '[object Function]';
    };
    var toInteger = function toInteger(value) {
      var number = Number(value);
      if (isNaN(number)) {
        return 0;
      }
      if (number === 0 || !isFinite(number)) {
        return number;
      }
      return (number > 0 ? 1 : -1) * Math.floor(Math.abs(number));
    };
    var maxSafeInteger = Math.pow(2, 53) - 1;
    var toLength = function toLength(value) {
      return Math.min(Math.max(toInteger(value), 0), maxSafeInteger);
    };

    // The length property of the from method is 1.
    return function from(arrayLike /*, mapFn, thisArg */) {
      // 1. Let C be the this value.
      var C = this;

      // 2. Let items be ToObject(arrayLike).
      var items = Object(arrayLike);

      // 3. ReturnIfAbrupt(items).
      if (arrayLike == null) {
        throw new TypeError('Array.from requires an array-like object - not null or undefined');
      }

      // 4. If mapfn is undefined, then let mapping be false.
      var mapFn = arguments.length > 1 ? arguments[1] : void undefined;
      var T = void 0;

      if (typeof mapFn !== 'undefined') {
        // 5. else
        // 5. a If IsCallable(mapfn) is false, throw a TypeError exception.
        if (!isCallable(mapFn)) {
          throw new TypeError('Array.from: when provided, the second argument must be a function');
        }

        // 5. b. If thisArg was supplied, let T be thisArg; else let T be undefined.
        if (arguments.length > 2) {
          T = arguments[2];
        }
      }

      // 10. Let lenValue be Get(items, "length").
      // 11. Let len be ToLength(lenValue).
      var len = toLength(items.length);

      // 13. If IsConstructor(C) is true, then
      // 13. a. Let A be the result of calling the [[Construct]] internal method
      // of C with an argument list containing the single item len.
      // 14. a. Else, Let A be ArrayCreate(len).
      var A = isCallable(C) ? Object(new C(len)) : new Array(len);

      // 16. Let k be 0.
      var k = 0;
      // 17. Repeat, while k < len… (also steps a - h)
      var kValue = void 0;
      while (k < len) {
        kValue = items[k];
        if (mapFn) {
          A[k] = typeof T === 'undefined' ? mapFn(kValue, k) : mapFn.call(T, kValue, k);
        } else {
          A[k] = kValue;
        }
        k += 1;
      }
      // 18. Let putStatus be Put(A, "length", len, true).
      A.length = len;
      // 20. Return A.
      return A;
    };
  }();
}

// https://tc39.github.io/ecma262/#sec-array.prototype.find
// Needed for IE support
if (!Array.prototype.find) {
  // eslint-disable-next-line no-extend-native
  Object.defineProperty(Array.prototype, 'find', {
    value: function value(predicate) {
      // 1. Let O be ? ToObject(this value).
      if (this == null) {
        throw new TypeError('"this" is null or not defined');
      }

      var o = Object(this);

      // 2. Let len be ? ToLength(? Get(O, "length")).
      var len = o.length >>> 0;

      // 3. If IsCallable(predicate) is false, throw a TypeError exception.
      if (typeof predicate !== 'function') {
        throw new TypeError('predicate must be a function');
      }

      // 4. If thisArg was supplied, let T be thisArg; else let T be undefined.
      var thisArg = arguments[1];

      // 5. Let k be 0.
      var k = 0;

      // 6. Repeat, while k < len
      while (k < len) {
        // a. Let Pk be ! ToString(k).
        // b. Let kValue be ? Get(O, Pk).
        // c. Let testResult be ToBoolean(? Call(predicate, T, « kValue, k, O »)).
        // d. If testResult is true, return kValue.
        var kValue = o[k];
        if (predicate.call(thisArg, kValue, k, o)) {
          return kValue;
        }
        // e. Increase k by 1.
        k++;
      }

      // 7. Return undefined.
      return undefined;
    }
  });
}

if (!Array.isArray) {
  Array.isArray = function (arg) {
    return Object.prototype.toString.call(arg) === '[object Array]';
  };
}

// Instance
var arrayIncludes = function arrayIncludes(array, value) {
  return array.indexOf(value) !== -1;
};

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var COLS_NUM = 12;

var GUTTER_TYPES = ['margin', 'padding'];

var LABEL_ALIGNMENTS = ['left', 'center', 'right'];

var FormGroup = {
  name: 'vbFormGroup',
  render: function render(h) {
    var label = h('label', {
      attrs: {
        for: this.labelFor
      },
      class: this.computedLabelClassNames,
      domProps: {
        innerHTML: this.label
      }
    });
    var labelCol = this.noLabelCol ? h(false) : h('div', {
      class: this.computedLabelColsClassNames
    }, [label]);

    var renderDescription = function () {
      if (this.$slots.description) {
        return this.$slots.description;
      }

      if (this.description && typeof this.description === 'string' && this.description.length) {
        return h('p', {
          class: ['help-text'],
          domProps: {
            innerHTML: this.description
          }
        });
      }
    }.bind(this);

    var renderInvalidFeedback = function () {
      if (this.$slots.invalidFeedback) {
        return this.$slots.invalidFeedback;
      }

      if (typeof this.invalidFeedback === 'string' && this.invalidFeedback.length) {
        return h('div', {
          class: ['form-error', 'is-visible'],
          domProps: {
            innerHTML: this.invalidFeedback
          }
        });
      }
    }.bind(this);

    var formControlCol = h('div', {
      class: this.computedInputColClassNames
    }, [this.$slots.default, this.state === false ? renderInvalidFeedback() : false, renderDescription()]);

    return h('div', {
      attrs: {
        id: this.safeId()
      },
      class: this.computedClass
    }, [labelCol, formControlCol]);
  },

  mixins: [IdMixin],
  props: {
    breakpoint: {
      type: String,
      default: 'medium'
    },
    description: {
      type: String
    },
    gutters: {
      type: String,
      default: null,
      validator: function validator(value) {
        return arrayIncludes(GUTTER_TYPES, value);
      }
    },
    horizontal: {
      type: Boolean,
      default: false
    },
    invalidFeedback: {
      type: String
    },
    label: {
      type: String,
      default: ''
    },
    labelClass: {
      type: [Array, String]
    },
    labelCols: {
      type: Number,
      default: 3
    },
    labelFor: {
      type: String,
      default: null
    },
    labelMiddle: {
      type: Boolean,
      default: false
    },
    labelTextAlign: {
      type: String,
      default: 'left',
      validator: function validator(value) {
        return arrayIncludes(LABEL_ALIGNMENTS, value);
      }
    },
    noLabelCol: {
      type: Boolean,
      default: false
    },
    state: {
      type: Boolean,
      default: true
    }
  },
  computed: {
    computedClass: function computedClass() {
      var classNames = ['grid-x'];

      switch (this.gutters) {
        case 'margin':
          classNames.push('grid-margin-x');
          break;
        case 'padding':
          classNames.push('grid-padding-x');
          break;
        case null:
        default:
          break;
      }

      return classNames;
    },
    computedInputColClassNames: function computedInputColClassNames() {
      var cols = void 0;

      if (this.noLabelCol) {
        cols = 12;
      } else {
        cols = this.horizontal ? COLS_NUM - this.labelCols : 12;
      }

      return ['cell', this.horizontal ? this.breakpoint + '-' + cols : 'small-' + COLS_NUM];
    },
    computedLabelClassNames: function computedLabelClassNames() {
      var classNames = ['text-' + this.labelTextAlign];

      if (Array.isArray(this.labelClass)) {
        classNames.push.apply(classNames, toConsumableArray(this.labelClass));
      }

      if (typeof this.labelClass === 'string') {
        classNames.push.apply(classNames, toConsumableArray(this.labelClass.split(' ')));
      }

      if (this.labelMiddle) {
        classNames.push('middle');
      }

      if (this.state === false) {
        classNames.push('is-invalid-label');
      }

      return classNames;
    },
    computedLabelColsClassNames: function computedLabelColsClassNames() {
      return ['cell', this.horizontal ? this.breakpoint + '-' + this.labelCols : 'small-12'];
    }
  }
};

var VuePlugin$2 = {
  install: function install(Vue) {
    registerComponent(Vue, 'vbFormGroup', FormGroup);
  }
};

vueUse(VuePlugin$2);

var FormControlStateMixin = {
  props: {
    state: {
      type: Boolean,
      default: null
    }
  },
  computed: {
    formControlStateClass: function formControlStateClass() {
      return this.state === false ? 'is-invalid-input' : null;
    }
  }
};

var FormMixin = {
  props: {
    disabled: {
      type: Boolean,
      default: false
    },
    name: {
      type: String,
      default: ''
    },
    required: {
      type: Boolean,
      default: false
    }
  }
};

var TYPES = ['text', 'date', 'datetime', 'datetime-local', 'email', 'month', 'number', 'password', 'search', 'tel', 'time', 'url', 'week'];

var vbFormInput = {
  render: function render(h) {
    return h('input', {
      attrs: {
        autocomplete: this.autocomplete,
        disabled: this.disabled,
        id: this.safeId(),
        name: this.name,
        placeholder: this.placeholder,
        readonly: this.readonly,
        required: this.required,
        type: this.type
      },
      class: this.computedClass,
      domProps: {
        value: this.localValue
      },
      on: {
        input: this.onInput,
        change: this.onChange
      },
      ref: 'input'
    });
  },

  mixins: [FormControlStateMixin, FormMixin, IdMixin],
  props: {
    autocomplete: {
      type: String
    },
    formatter: {
      type: Function
    },
    placeholder: {
      type: String,
      default: ''
    },
    readonly: {
      type: Boolean,
      default: false
    },
    type: {
      type: String,
      default: 'text',
      validator: function validator(value) {
        return arrayIncludes(TYPES, value);
      }
    },
    value: {
      default: null
    }
  },
  data: function data() {
    return {
      localValue: this.value
    };
  },

  computed: {
    computedClass: function computedClass() {
      return [this.formControlStateClass];
    }
  },
  methods: {
    format: function format(value, event) {
      if (this.formatter) {
        var formattedValue = this.formatter(value, event);

        if (formattedValue !== value) {
          return formattedValue;
        }
      }

      return value;
    },
    onChange: function onChange(event) {
      this.localValue = this.format(event.target.value, event);

      this.$emit('change', this.localValue);
    },
    onInput: function onInput(event) {
      var value = event.target.value;

      this.localValue = this.format(value, event);
    }
  },
  watch: {
    value: function value(current, prev) {
      if (current !== prev) {
        this.localValue = current;
      }
    },
    localValue: function localValue(current, prev) {
      if (current !== prev) {
        this.$emit('input', current);
      }
    }
  }
};

var VuePlugin$3 = {
  install: function install(Vue) {
    registerComponent(Vue, 'vbFormInput', vbFormInput);
  }
};

vueUse(VuePlugin$3);

function isObject(obj) {
  return obj && {}.toString.call(obj) === '[object Object]';
}

function mapObjectToOption(obj, vm) {
  return {
    disabled: obj[vm.disabledProp],
    label: obj[vm.labelProp],
    value: obj[vm.valueProp]
  };
}

function mapValueToOption(val) {
  return {
    disabled: false,
    label: String(val),
    value: val
  };
}

var renderOption = function renderOption(h, option, index) {
  return h('option', {
    attrs: {
      disabled: Boolean(option.disabled)
    },
    domProps: {
      innerHTML: option.label,
      value: option.value
    },
    key: 'option_' + index + '_opt'
  });
};

var FormSelect = {
  render: function render(h) {
    var options = this.computedOptions.map(function (option, index) {
      return renderOption(h, option, index);
    });

    return h('select', {
      attrs: {
        'aria-invalid': this.computedAriaInvalid,
        'aria-required': this.required ? 'true' : null,
        disabled: this.disabled,
        id: this.safeId(),
        multiple: this.multiple || null,
        name: this.name,
        required: this.required,
        size: this.computedSelectSize
      },
      class: this.formControlStateClass,
      directives: [{
        name: 'model',
        rawName: 'v-model',
        value: this.localValue,
        expression: 'localValue'
      }],
      on: {
        change: this.onChange
      },
      ref: 'input'
    }, [this.$slots.first, options, this.$slots.default]);
  },

  mixins: [FormControlStateMixin, FormMixin, IdMixin],
  props: {
    ariaInvalid: {
      type: [Boolean, String],
      default: false
    },
    disabledProp: {
      type: String,
      default: 'disabled'
    },
    labelProp: {
      type: String,
      default: 'label'
    },
    multiple: {
      type: Boolean,
      default: false
    },
    size: {
      type: Number,
      default: null
    },
    options: {
      type: Array,
      default: function _default() {
        return [];
      }
    },
    value: {},
    valueProp: {
      type: String,
      default: 'value'
    }
  },
  data: function data() {
    return {
      localValue: this.value
    };
  },

  computed: {
    computedAriaInvalid: function computedAriaInvalid() {
      if (this.ariaInvalid === true || this.ariaInvalid === 'true') {
        return 'true';
      }
      return this.stateClass === 'is-invalid' ? 'true' : null;
    },
    inputClass: function inputClass() {
      return [this.formControlStateClass];
    },
    computedOptions: function computedOptions() {
      var options = this.options;
      var vm = this;

      if (Array.isArray(options)) {
        return options.map(function (o) {
          return isObject(o) ? mapObjectToOption(o, vm) : mapValueToOption(o);
        });
      }
    },
    computedSelectSize: function computedSelectSize() {
      return this.size === 0 ? null : this.size;
    }
  },
  methods: {
    onChange: function onChange(event) {
      var selectedVal = Array.from(event.target.options).filter(function (o) {
        return o.selected;
      }).map(function (o) {
        return '_value' in o ? o._value : o.value;
      });
      this.localValue = event.target.multiple ? selectedVal : selectedVal[0];
      this.$emit('change', this.localValue);
    }
  },
  watch: {
    value: function value(current) {
      this.localValue = current;
    },
    localValue: function localValue() {
      this.$emit('input', this.localValue);
    }
  }
};

var VuePlugin$4 = {
  install: function install(Vue) {
    registerComponent(Vue, 'vbFormInput', FormSelect);
  }
};

vueUse(VuePlugin$4);

var __assign = Object.assign || function (e) {
  for (var a, s = 1, t = arguments.length; s < t; s++) {
    for (var r in a = arguments[s]) {
      Object.prototype.hasOwnProperty.call(a, r) && (e[r] = a[r]);
    }
  }return e;
};function mergeData() {
  for (var e, a, s = {}, t = arguments.length; t--;) {
    for (var r = 0, c = Object.keys(arguments[t]); r < c.length; r++) {
      switch (e = c[r]) {case "class":case "style":case "directives":
          Array.isArray(s[e]) || (s[e] = []), s[e] = s[e].concat(arguments[t][e]);break;case "staticClass":
          if (!arguments[t][e]) break;void 0 === s[e] && (s[e] = ""), s[e] && (s[e] += " "), s[e] += arguments[t][e].trim();break;case "on":case "nativeOn":
          s[e] || (s[e] = {});for (var o = 0, n = Object.keys(arguments[t][e] || {}); o < n.length; o++) {
            a = n[o], s[e][a] ? s[e][a] = [].concat(s[e][a], arguments[t][e][a]) : s[e][a] = arguments[t][e][a];
          }break;case "attrs":case "props":case "domProps":case "scopedSlots":case "staticStyle":case "hook":case "transition":
          s[e] || (s[e] = {}), s[e] = __assign({}, arguments[t][e], s[e]);break;case "slot":case "key":case "ref":case "tag":case "show":case "keepAlive":default:
          s[e] || (s[e] = arguments[t][e]);}
    }
  }return s;
}

/**
 * Aliasing Object[method] allows the minifier to shorten methods to a single character variable,
 * as well as giving BV a chance to inject polyfills.
 * As long as we avoid
 * - import * as Object from "utils/object"
 * all unused exports should be removed by tree-shaking.
 */

// @link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/assign
if (typeof Object.assign !== 'function') {
  Object.assign = function (target, varArgs) {
    // .length of function is 2

    if (target == null) {
      // TypeError if undefined or null
      throw new TypeError('Cannot convert undefined or null to object');
    }

    var to = Object(target);

    for (var index = 1; index < arguments.length; index++) {
      var nextSource = arguments[index];

      if (nextSource != null) {
        // Skip over if undefined or null
        for (var nextKey in nextSource) {
          // Avoid bugs when hasOwnProperty is shadowed
          if (Object.prototype.hasOwnProperty.call(nextSource, nextKey)) {
            to[nextKey] = nextSource[nextKey];
          }
        }
      }
    }
    return to;
  };
}

// @link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is#Polyfill
if (!Object.is) {
  Object.is = function (x, y) {
    // SameValue algorithm
    if (x === y) {
      // Steps 1-5, 7-10
      // Steps 6.b-6.e: +0 != -0
      return x !== 0 || 1 / x === 1 / y;
    } else {
      // Step 6.a: NaN == NaN
      // eslint-disable-next-line no-self-compare
      return x !== x && y !== y;
    }
  };
}

var assign = Object.assign;

/**
 * The Link component is used in many other BV components.
 * As such, sharing its props makes supporting all its features easier.
 * However, some components need to modify the defaults for their own purpose.
 * Prefer sharing a fresh copy of the props to ensure mutations
 * do not affect other component references to the props.
 *
 * https://github.com/vuejs/vue-router/blob/dev/src/components/link.js
 * @return {{}}
 */
function propsFactory() {
  return {
    href: {
      type: String,
      default: null
    },
    rel: {
      type: String,
      default: null
    },
    target: {
      type: String,
      default: '_self'
    },
    active: {
      type: Boolean,
      default: false
    },
    activeClass: {
      type: String,
      default: 'active'
    },
    append: {
      type: Boolean,
      default: false
    },
    disabled: {
      type: Boolean,
      default: false
    },
    event: {
      type: [String, Array],
      default: 'click'
    },
    exact: {
      type: Boolean,
      default: false
    },
    exactActiveClass: {
      type: String,
      default: 'active'
    },
    replace: {
      type: Boolean,
      default: false
    },
    routerTag: {
      type: String,
      default: 'a'
    },
    to: {
      type: [String, Object],
      default: null
    }
  };
}

/**
 * Shamelessly ripped off of VueBootstrap's <b-link> component.
 * @see https://github.com/bootstrap-vue/bootstrap-vue/blob/v2.0.0-rc.6/src/components/link/link.js
 */

function propsFactory$1() {
  var props$$1 = propsFactory();

  // Assign Foundation-specific CSS class names.
  props$$1.activeClass.default = 'is-active';
  props$$1.exactActiveClass.default = 'is-active';

  return props$$1;
}

var props$1 = propsFactory$1();

function computeTag$1(props$$1, parent) {
  return parent.$router && props$$1.to && !props$$1.disabled ? 'router-link' : 'a';
}

function computeRel$1(_ref) {
  var target = _ref.target,
      rel = _ref.rel;

  if (target === '_blank' && rel === null) {
    return 'noopener';
  }
  return rel || null;
}

function computeHref$1(_ref2, tag) {
  var disabled = _ref2.disabled,
      href = _ref2.href,
      to = _ref2.to;

  if (tag === 'router-link') {
    return;
  }

  if (href) {
    return href;
  }

  if (to) {
    if (typeof to === 'string') {
      return to;
    }

    if ((typeof to === 'undefined' ? 'undefined' : _typeof(to)) === 'object' && typeof to.path === 'string') {
      return to.path;
    }
  }

  return '#';
}

function clickHandlerFactory$1(_ref3) {
  var disabled = _ref3.disabled,
      tag = _ref3.tag,
      href = _ref3.href,
      suppliedHandler = _ref3.suppliedHandler,
      parent = _ref3.parent;

  var isRouterLink = tag === 'router-link';

  return function onClick(e) {
    if (disabled && e instanceof Event) {
      // Stop event from bubbling up.
      e.stopPropagation();
      // Kill the event loop attached to this specific EventTarget.
      e.stopImmediatePropagation();
    } else {
      parent.$root.$emit('vb::link::clicked', e);

      if (isRouterLink && e.target.__vue__) {
        e.target.__vue__.$emit('click', e);
      }

      if (typeof suppliedHandler === 'function') {
        suppliedHandler.apply(undefined, arguments);
      }
    }

    if (!isRouterLink && href === '#' || disabled) {
      // Stop scroll-to-top behavior or navigation.
      e.preventDefault();
    }
  };
}

var Link = {
  functional: true,
  props: propsFactory$1(),
  render: function render(h, _ref4) {
    var props$$1 = _ref4.props,
        data = _ref4.data,
        parent = _ref4.parent,
        children = _ref4.children;

    var tag = computeTag$1(props$$1, parent);
    var href = computeHref$1(props$$1, tag);
    var eventType = tag === 'router-link' ? 'nativeOn' : 'on';
    var suppliedHandler = (data[eventType] || {}).click;
    var handlers = {
      click: clickHandlerFactory$1({ tag: tag, href: href, disabled: props$$1.disabled, suppliedHandler: suppliedHandler, parent: parent })
    };

    var componentData = mergeData(data, {
      class: [props$$1.active ? props$$1.exact ? props$$1.exactActiveClass : props$$1.activeClass : null, props$$1.disabled ? 'disabled' : null],
      attrs: {
        rel: computeRel$1(props$$1),
        href: href,
        target: props$$1.target,
        tabindex: props$$1.disabled ? '-1' : data.attrs ? data.attrs.tabindex : null,
        'aria-disabled': tag === 'a' && props$$1.disabled ? 'true' : null
      },
      props: assign(props$$1, {
        tag: props$$1.routerTag
      })
    });

    // If href prop exists on router-link (even undefined or null) it fails working on SSR
    if (!componentData.attrs.href) {
      delete componentData.attrs.href;
    }

    // We want to overwrite any click handler since our callback
    // will invoke the supplied handler if !props.disabled
    componentData[eventType] = assign(componentData[eventType] || {}, handlers);

    return h(tag, componentData, children);
  }
};

var VuePlugin$5 = {
  install: function install(Vue) {
    registerComponent(Vue, 'vbLink', Link);
  }
};

vueUse(VuePlugin$5);

var ALIGNMENTS = ['left', 'center', 'right'];

function computeClassNames(props) {
  return {
    dropdown: props.dropdown,
    expanded: props.expanded,
    simple: props.simple,
    vertical: props.vertical,

    'align-center': props.align === 'center',
    'align-right': props.align === 'right'
  };
}

var vbMenu = {
  functional: true,
  render: function render(h, _ref) {
    var children = _ref.children,
        data = _ref.data,
        props = _ref.props;

    var componentData = {
      staticClass: 'menu',
      class: computeClassNames(props)
    };

    return h('ul', mergeData(data, componentData), children);
  },

  props: {
    align: {
      type: String,
      default: 'left',
      validator: function validator(value) {
        return arrayIncludes(ALIGNMENTS, value);
      }
    },
    dropdown: {
      type: Boolean,
      default: false
    },
    expanded: {
      type: Boolean,
      default: false
    },
    simple: {
      type: Boolean,
      default: false
    },
    vertical: {
      type: Boolean,
      default: false
    }
  }
};

var props$2 = propsFactory$1();

var vbMenuItem = {
  functional: true,
  props: props$2,
  render: function render(h, _ref) {
    var props = _ref.props,
        data = _ref.data,
        children = _ref.children;

    return h('li', data, [h(Link, { props: props }, children)]);
  }
};

var vbMenuText = {
  functional: true,
  props: {
    tag: {
      type: String,
      default: 'span'
    }
  },
  render: function render(h, _ref) {
    var props = _ref.props,
        children = _ref.children;

    return h('li', {
      staticClass: 'menu-text'
    }, [h(props.tag, [children])]);
  }
};

var VuePlugin$6 = {
  install: function install(Vue) {
    registerComponent(Vue, 'vbMenu', vbMenu);
    registerComponent(Vue, 'vbMenuItem', vbMenuItem);
    registerComponent(Vue, 'vbMenuText', vbMenuText);
  }
};

vueUse(VuePlugin$6);

// Threshold of limit size when we start/stop showing ellipsis
var ELLIPSIS_THRESHOLD = 2;

var props$3 = {
  centered: {
    type: Boolean,
    default: false
  },
  disabled: {
    type: Boolean,
    default: false
  },
  ellipsisText: {
    type: String,
    default: '&hellip;'
  },
  firstText: {
    type: String,
    default: 'First'
  },
  hideEllipsis: {
    type: Boolean,
    default: false
  },
  hideGoToEndButtons: {
    type: Boolean,
    default: false
  },
  lastText: {
    type: String,
    default: 'Last'
  },
  limit: {
    type: Number,
    default: 5,
    validator: function validator(limit) {
      return limit % 2 !== 0;
    }
  },
  nextText: {
    type: String,
    default: 'Next'
  },
  perPage: {
    type: Number,
    default: 20
  },
  prevText: {
    type: String,
    default: 'Previous'
  },
  totalRows: {
    type: Number,
    default: 20
  },
  value: {
    type: Number,
    default: 1
  }

  /**
   * First and last page button factory.
   * @param dir
   * @return {VNode}
   */
};function makeExtremes(dir) {
  var vm = this;
  var h = this.$createElement;
  var pageNum = dir === 'first' ? 1 : this.numberOfPages;
  var disabled = this.disabled || this.isActive(pageNum);

  var childAttrs = {
    domProps: {
      innerHTML: dir === 'first' ? this.firstText : this.lastText
    }
  };

  var child = disabled ? h('span', childAttrs) : h('a', merge({}, childAttrs, {
    attrs: this.linkProps(),
    on: {
      click: function click(event) {
        vm.onClick(pageNum, event);
      }
    }
  }));

  return h('li', {
    class: {
      'pagination-first': dir === 'first',
      'pagination-last': dir === 'last',
      'disabled': disabled
    }
  }, [child]);
}

/**
 * Prev and next page button factory.
 * @param dir
 * @return {VNode}
 */
function makePrevOrNext(dir) {
  var vm = this;
  var h = this.$createElement;
  var pageNum = dir === 'prev' ? this.prevPage : this.nextPage;
  var disabled = this.disabled || pageNum === null;

  var childAttrs = {
    domProps: {
      innerHTML: dir === 'prev' ? this.prevText : this.nextText
    }
  };

  var child = disabled ? h('span', childAttrs) : h('a', merge({}, childAttrs, {
    attrs: this.linkProps(),
    on: {
      click: function click(event) {
        vm.onClick(pageNum, event);
      }
    }
  }));

  return h('li', {
    class: {
      'pagination-previous': dir === 'prev',
      'pagination-next': dir === 'next',
      'disabled': disabled
    }
  }, [child]);
}

function makeEllipsis() {
  return this.$createElement('li', {
    attrs: {
      'aria-hidden': true
    },
    class: ['ellipsis']
  });
}

function makePageButton(pageNum) {
  var vm = this;
  var h = this.$createElement;
  var isCurrent = this.isActive(pageNum);
  var children = [];

  if (isCurrent) {
    children.push(h('span', {
      class: 'show-for-sr',
      domProps: {
        innerHTML: 'You\'re on page '
      }
    }));
    children.push(h('span', {
      domProps: {
        innerHTML: pageNum
      }
    }));
  } else {
    children.push(h('a', {
      attrs: {
        'aria-label': 'Page ' + pageNum,
        href: '#'
      },
      domProps: {
        innerHTML: pageNum
      },
      on: {
        click: function click(event) {
          vm.onClick(pageNum, event);
        }
      }
    }));
  }

  return h('li', {
    class: {
      current: isCurrent
    }
  }, children);
}

var vbPagination = {
  name: 'vbPagination',
  render: function render(h) {
    var _this = this;

    var buttons = [];

    buttons.push(this.hideGoToEndButtons ? h(false) : makeExtremes.call(this, 'first'));
    buttons.push(this.hideGoToEndButtons ? h(false) : makePrevOrNext.call(this, 'prev'));
    buttons.push(this.showFirstDots ? makeEllipsis.call(this) : h(false));

    this.pageList.forEach(function (pageNum) {
      buttons.push(makePageButton.call(_this, pageNum));
    }, this);

    buttons.push(this.showLastDots ? makeEllipsis.call(this) : h(false));
    buttons.push(this.hideGoToEndButtons ? h(false) : makePrevOrNext.call(this, 'next'));
    buttons.push(this.hideGoToEndButtons ? h(false) : makeExtremes.call(this, 'last'));

    var listProps = {
      attrs: {
        role: 'menubar'
      },
      class: {
        pagination: true,
        'text-center': this.centered,
        'vb-pagination': true
      }
    };

    return h('ul', listProps, [buttons]);
  },

  props: props$3,
  data: function data() {
    return {
      currentPage: this.value
    };
  },

  computed: {
    nextPage: function nextPage() {
      return this.currentPage === this.numberOfPages ? null : this.currentPage + 1;
    },
    numberOfPages: function numberOfPages() {
      var result = Math.ceil(this.totalRows / this.perPage);

      return result < 1 ? 1 : result;
    },
    pageList: function pageList() {
      var _this2 = this;

      var pages = [];
      var limit = this.limit > this.numberOfPages ? this.numberOfPages : this.limit;

      var start = this.currentPage - limit;
      var end = this.currentPage + limit;
      var perSide = Math.floor((limit - 1) / 2);

      for (var i = start; i < this.currentPage; i++) {
        pages.push(i);
      }

      pages.push(this.currentPage);

      for (var _i = this.currentPage + 1; _i <= end; _i++) {
        pages.push(_i);
      }

      var startIndex = pages.findIndex(function (page) {
        return page === _this2.currentPage - perSide;
      });

      while (pages[startIndex] < 1) {
        startIndex++;
      }

      var endIndex = startIndex + limit;

      while (pages[endIndex] > this.numberOfPages + 1) {
        startIndex--;
        endIndex--;
      }

      return pages.slice(startIndex, endIndex);
    },
    prevPage: function prevPage() {
      return this.currentPage === 1 ? null : this.currentPage - 1;
    },
    showFirstDots: function showFirstDots() {
      if (this.hideEllipsis) return false;

      var numPagesBeforeCurrent = this.currentPage - 1;

      return numPagesBeforeCurrent > ELLIPSIS_THRESHOLD;
    },
    showLastDots: function showLastDots() {
      if (this.hideEllipsis) return false;

      var numPagesAfterCurrent = this.numberOfPages - this.currentPage;

      return numPagesAfterCurrent > ELLIPSIS_THRESHOLD;
    }
  },
  methods: {
    isActive: function isActive(pageNum) {
      return pageNum === this.currentPage;
    },
    linkProps: function linkProps() {
      return {
        href: '#'
      };
    },
    normalizeCurrentPage: function normalizeCurrentPage(value) {
      if (value < 1) {
        return 1;
      }

      if (value > this.numberOfPages) {
        return this.numberOfPages;
      }

      return value;
    },
    onClick: function onClick(num, event) {
      var _this3 = this;

      event.preventDefault();

      if (num > this.numberOfPages) {
        num = this.numberOfPages;
      } else if (num < 1) {
        num = 1;
      }

      this.currentPage = num;

      this.$nextTick(function () {
        var target = event.target;
        target.focus();
        _this3.$emit('change', _this3.currentPage);
      });
    }
  },
  created: function created() {
    this.currentPage = this.normalizeCurrentPage(this.currentPage);
  }
};

var VuePlugin$7 = {
  install: function install(Vue) {
    registerComponent(Vue, 'vbPagination', vbPagination);
  }
};

vueUse(VuePlugin$7);



var components = /*#__PURE__*/Object.freeze({
  ButtonClose: VuePlugin,
  Callout: VuePlugin$1,
  FormGroup: VuePlugin$2,
  FormInput: VuePlugin$3,
  FormSelect: VuePlugin$4,
  Link: VuePlugin$5,
  Menu: VuePlugin$6,
  Pagination: VuePlugin$7
});

var VuePlugin$8 = {
  install: function install(Vue) {
    if (Vue._vue_bedrock_installed) {
      return;
    }

    Vue._vue_bedrock_installed = true;

    // Register component plugins
    for (var plugin in components) {
      Vue.use(components[plugin]);
    }
  }
};

vueUse(VuePlugin$8);

export default VuePlugin$8;
//# sourceMappingURL=vue-bedrock.esm.js.map
